#version 450 core

in vec2 texCoords;

layout(location=0) out vec4 color;

struct Material {
	sampler2D texture_diffuse1; // actually colormap
	sampler2D texture_specular1; // actually depthmap
};

uniform Material material;
uniform bool horizontal = true;
uniform float near = 0.1;
uniform float far = 5000;

float[] weights = { 1, 6, 15, 20, 15, 6, 1 };
//float[] weights = { 1, 1 };

float linearDepth(vec2 uv) {
	float d = texture(material.texture_specular1, uv).r;
	return (near * far) / (far + d * (near - far));
}

// applies a blur on a 7 pixel window
vec3 blur(vec2 uv) {
	const int width = weights.length();
	//vec2 invColorTexSize = 1 / textureSize(material.texture_diffuse1, 0);
	//vec2 invDepthTexSize = 1 / textureSize(material.texture_specular1, 0);
	//vec2 direction = vec2(2); //horizontal ? vec2(1, 0) : vec2(0, 1);
	//float centerDepth = linearDepth(uv).r;
	//float denom = 0;
	//vec3 result = vec3(0);
	//for (int i = 0; i < width; i++) {
	//	int texelOffset = i - width / 2;
	//	vec2 colorOffset = direction * texelOffset * invColorTexSize;
	//	vec2 depthOffset = direction * texelOffset * invDepthTexSize;
	//	vec3 color = texture(material.texture_diffuse1, uv + colorOffset).rgb;
	//	float depth = linearDepth(uv + depthOffset).r;
	//	float depthDiff = depth - centerDepth;
	//	float w = weights[i];
	//	denom += w;
	//	result += color * w;
	//}
	//return result / denom;
	//return result / 2;

	vec2 TexCoords = uv;
	vec2 colorTexSize = 1.0 / textureSize(material.texture_diffuse1, 0); 
	vec2 depthTexSize = 1.0 / textureSize(material.texture_specular1, 0); 
		vec2 direction = horizontal ? vec2(1, 0) : vec2(0, 1);
	float denom = 0;
	float centerDepth = linearDepth(uv);
    vec3 result = vec3(0); 
	for(int i = 1; i < width; ++i)
	{
		vec2 colorTexOffset	= colorTexSize * i * direction;
		vec2 depthTexOffset	= depthTexSize * i * direction;
		vec3 color = texture(material.texture_diffuse1, TexCoords + colorTexOffset).rgb;

		float depthDiff = linearDepth(uv + depthTexOffset) - centerDepth;
		float weight = weights[i] * abs(depthDiff) * weights[i];
		denom += weight;
		result += color * weight;
	}
    return result / denom;
}

void main() {
	vec2 uv = texCoords;
	uv.y = uv.y * -1 + 1;
	color.rgb = blur(uv);
}