#version 450 core
layout (location = 0) in vec3 aPos;
layout (location = 1) in vec3 aNormal;
layout (location = 2) in vec2 aTexCoords;
layout (location = 3) in vec2 instancePosition;

struct Material {
	sampler2D texture_diffuse1;
	sampler2D texture_specular1;
	sampler2D texture_depth1;
	float shininess;
};
uniform Material material;

uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;
uniform mat4 lightSpaceMatrix;
uniform mat4 terrainMatrix;
uniform sampler2D heightmap;

//out VS_FS_INTERFACE {
out vec3 worldPos;
out vec4 lightSpacePos;
out vec2 texCoords;
out vec3 normal;
//} vertex;

void main() {

	// shift instance on xz-plane
	float height = texture(material.texture_depth1, instancePosition).r;
	float test = texture(material.texture_diffuse1, instancePosition).r;
	float test2 = texture(material.texture_specular1, instancePosition).r;
	vec4 instanceOffset = terrainMatrix * vec4(instancePosition.x, height, instancePosition.y, 1) + (test + test2) * 0.00001;
	mat4 instanceTranslation = mat4(1);
	instanceTranslation[0][0] = instanceOffset.x;
	instanceTranslation[2][2] = instanceOffset.z;
	gl_Position = projection * view * model * vec4(aPos + vec3(instanceOffset.x, 0, instanceOffset.z), 1.0);
	texCoords = aTexCoords;

	// fragment position to world- and lightspace
    worldPos = vec3(model * vec4(aPos , 1.0));
	lightSpacePos = lightSpaceMatrix * vec4(worldPos, 1.0); 

	// normal direction to worldspace
	mat3 invTransModel = mat3(transpose(inverse(model)));
	normal = invTransModel * aNormal;
};