#version 440 core

layout (vertices=4) out;

in vec2 TexCoord[];
out vec2 TextureCoord[];

uniform mat4 model;
uniform mat4 view;
uniform vec3 CameraPos;

void main()
{
    // pass through attributes
    gl_out[gl_InvocationID].gl_Position = gl_in[gl_InvocationID].gl_Position;
    TextureCoord[gl_InvocationID] = TexCoord[gl_InvocationID];

    if(gl_InvocationID == 0)
    {
        const int MIN_TESS_LEVEL = 1;
        const int MAX_TESS_LEVEL = 64;
        const float MIN_DISTANCE = 500;
        const float MAX_DISTANCE = 5000;

        // ----------------------------------------------------------------------
        // Step 2: transform each vertex into world-space
        vec4 worldSpacePos00 = model * gl_in[0].gl_Position;
        vec4 worldSpacePos01 = model * gl_in[1].gl_Position;
        vec4 worldSpacePos10 = model * gl_in[2].gl_Position;
        vec4 worldSpacePos11 = model * gl_in[3].gl_Position;
        
        // distance to camera scaled between 0 and 1
        float distance00 = clamp((distance(CameraPos, worldSpacePos00.xyz) - MIN_DISTANCE) / (MAX_DISTANCE - MIN_DISTANCE), 0, 1);
        float distance01 = clamp((distance(CameraPos, worldSpacePos01.xyz) - MIN_DISTANCE) / (MAX_DISTANCE - MIN_DISTANCE), 0, 1);
        float distance10 = clamp((distance(CameraPos, worldSpacePos10.xyz) - MIN_DISTANCE) / (MAX_DISTANCE - MIN_DISTANCE), 0, 1);
        float distance11 = clamp((distance(CameraPos, worldSpacePos11.xyz) - MIN_DISTANCE) / (MAX_DISTANCE - MIN_DISTANCE), 0, 1);

        // scale distance exponentially
        distance00 = 1 - exp(-2 * distance00);
        distance01 = 1 - exp(-2 * distance01);
        distance10 = 1 - exp(-2 * distance10);
        distance11 = 1 - exp(-2 * distance11);

        // interpolate edge tessellation level based on closer vertex
        float tessLevel0 = mix( MAX_TESS_LEVEL, MIN_TESS_LEVEL, min(distance10, distance00) );
        float tessLevel1 = mix( MAX_TESS_LEVEL, MIN_TESS_LEVEL, min(distance00, distance01) );
        float tessLevel2 = mix( MAX_TESS_LEVEL, MIN_TESS_LEVEL, min(distance01, distance11) );
        float tessLevel3 = mix( MAX_TESS_LEVEL, MIN_TESS_LEVEL, min(distance11, distance10) );

        // set the corresponding outer edge tessellation levels
        gl_TessLevelOuter[0] = tessLevel0;
        gl_TessLevelOuter[1] = tessLevel1;
        gl_TessLevelOuter[2] = tessLevel2;
        gl_TessLevelOuter[3] = tessLevel3;

        // set the inner tessellation levels to the max of the two parallel edges
        gl_TessLevelInner[0] = max(tessLevel1, tessLevel3);
        gl_TessLevelInner[1] = max(tessLevel0, tessLevel2);
    }
}