#version 450 core

in vec2 texCoords;

layout(location=0) out vec4 color;

struct Material {
	sampler2D texture_diffuse1; // actually low-res colormap
	sampler2D texture_specular1; // actually low-res depthmap
	sampler2D depth_map1; // actually high-res depthmap
};

uniform Material material;

// TODO: use uniforms for clipping planes
float linear_depth(float d) {
	float near = 0.1;
	float far = 4000.0;
	return (near * far) / (far + d * (near - far));
}
vec4 linear_depth4(vec4 d) {
	float near = 0.1;
	float far = 5000.0;
	return (near * far) / (far + d * (near - far));
}

// https://developer.download.nvidia.com/assets/gamedev/files/sdk/11/OpacityMappingSDKWhitePaper.pdf
void main() {

	// get a sample from the high-res depth and four samples from the low-res depth
	float high_res_depth = linear_depth(texture(material.depth_map1, texCoords).x);
	vec4 low_res_depths = linear_depth4(textureGather(material.texture_specular1, texCoords, 0));

	// calculate absolute depth differences
	color.rgb = vec3(0);
	int nearest_i = 0;
	float nearest = 9999999999.0;
	vec4 diffs = abs(low_res_depths - high_res_depth);
	float abs_diff = dot(diffs, vec4(1.0));
	
	// sample bilinearily if the absolute depth difference is below threshold ...
	if (abs_diff < 10.0) {
		color = texture(material.texture_diffuse1, texCoords);
		return;
	}

	// ... else calculate the nearest depth position and sample color from there
	for (int i = 0; i < 4; i++) {
		if (diffs[i] < nearest) {
			nearest_i = i;
			nearest = diffs[i];
		}
	}
	if (nearest_i == 0) {
		color = textureOffset(material.texture_diffuse1, texCoords, ivec2(0, 1));
	} else if (nearest_i == 1) {
		color = textureOffset(material.texture_diffuse1, texCoords, ivec2(1, 1));
	} else if (nearest_i == 2) {
		color = textureOffset(material.texture_diffuse1, texCoords, ivec2(1, 0));
	} else {
		color = textureOffset(material.texture_diffuse1, texCoords, ivec2(0, 0));
	}
}
